/*

 Flash XML Compressor - Java version 
	
 by Strille, 2004

 Home Page: http://www.strille.net/tutorials/FlashXMLCompressor/

*/

import java.io.*; 


public class FlashXMLCompressor {
	
	private static String encode(String m) {
		String outComp = m.substring(0, 1);
		String encodeChar = null;
		
		for (int n=161;n<191;n++) { //- Find a character not in the file to use in the encoding formatting
			String c = new Character((char)n).toString();
			
			if (m.indexOf(c) == -1) {
				encodeChar = c;
				break;
			}
		}
		if (encodeChar == null) {
			System.out.println("Error: This file can't be compressed (No free character found)");
			return "";
		}
		
		int radix = 114;
		int bufferSize = radix*radix-1;
		int currPos = 1;
		int codingBufferSize = radix-1;
		String codingA = m.substring(currPos, Math.min(m.length(), currPos+codingBufferSize));
		
		while (codingA.length() > 0) {
			int res = encodeFindBest(m, Math.max(currPos-bufferSize, 0), currPos-1, codingA);
			if (res == -1) {
				outComp += m.charAt(currPos);
				currPos++;
			} else {
				int p = res/1000;
				int l = res-p*1000;
				p = currPos-p;
				int ms = (int)(p/radix);
				int ls = p-ms*radix;
				outComp += encodeChar + new Character((char)(14+ms)).toString() + new Character((char)(14+ls)).toString() + new Character((char)(14+l)).toString();
				currPos += l;
			}
			
			codingA = m.substring(currPos, Math.min(m.length(), currPos+codingBufferSize));
		}
		
		return "v1.0 " + encodeChar + outComp;
	}
	
	
	private static int encodeFindBest(String buf, int winS, int winE, String cA) {
		if (winE > buf.length()) {
			winE = buf.length();
		}
		
		int lookAheadLength = cA.length();
		String winStr = buf.substring(winS, winE);
		int sequenceLength = 5;
		int matchLength = 0;
		int matchPos = 0;
		int n;
		
		while (sequenceLength <= lookAheadLength) 
		{
			n = winStr.indexOf(cA.substring(0, sequenceLength));
			if (n == -1) 
			{
				break;
			}
			else 
			{
				matchLength = sequenceLength;
				matchPos = n;
			}
			sequenceLength++;
		}
		
		if (matchLength > 0) 
		{
			return 1000*(winS+matchPos) +  matchLength;
		}
		else 
		{
			return -1;
		}		
	}
	
	
	private static String loadFile(String inputFileName) {
		String inputText = "";
		
		try {
			FileReader inputFileReader = new FileReader(inputFileName);
			BufferedReader inputStream = new BufferedReader(inputFileReader);
			
			String inLine = null;
			while ((inLine = inputStream.readLine()) != null) {
				inputText += inLine;
			}
				
			// Strip tabs
			int tabPos = -1;
			while ((tabPos = inputText.indexOf("\t")) != -1) {
				inputText = inputText.substring(0, tabPos) + inputText.substring(tabPos+1, inputText.length());
			}

			inputStream.close();
		} catch (IOException e) {
			System.out.println("Error: " + e.getMessage());
		}
		
		return inputText;
	}
	

	private static void saveFile(String txt, String fileName) {
		try {
			FileWriter outputFileReader  = new FileWriter(fileName);
			PrintWriter outputStream  = new PrintWriter(outputFileReader);
			outputStream.print(txt);
			outputStream.close();
		} catch (IOException e) {
			System.out.println("Error: " + e.getMessage());
		}
	}
	
	
	public static void main (String args[]) {
				
		if (args.length > 0) {
			String fileName = args[0];
			String outFile;
				
			if (args.length == 1) {
				if (fileName.lastIndexOf(".") != -1) {
					outFile = fileName.substring(0, fileName.lastIndexOf(".")) + "Compressed" + fileName.substring(fileName.lastIndexOf("."), fileName.length());
				} else {
					outFile = fileName + "Compressed";
				}
			} else {
				outFile = args[1];
			}
			
			String text = loadFile(fileName);
			
			if (text.length() > 0) {
				System.out.println("Encoding " + fileName + "...");
				String encoded = encode(text);
				saveFile(encoded, outFile);
				
				double percentage = 100*encoded.length()/text.length();
				System.out.println("Compressed file: " + outFile + " (" + percentage + "% of the original)");
			}
		} else {
			System.out.println("No file name specified!");
			System.out.println("Usage: java FlashXMLCompressor <source filename> <optional: new filename>");
		}
		
		System.out.print("\nPress Return to exit\n");
		try {
			System.in.read();
		} catch (Exception e) {
			System.out.println("Error: " + e.toString());
		}
	}
}

